import type { ON_GLOBAL_RESPONSE_PROGRESS, ABORT_CONTROLLER } from '../constants';
import type Req from '../Req';
import type Res from '../Res';
export interface ResInit extends ResponseInit {
    meta?: Record<string, any>;
    timeout?: number;
    responseType?: ResponseType;
    throwHttpError?: boolean;
    [ABORT_CONTROLLER]?: AbortController;
    onResponseProgress?: ProgressCallback;
    [ON_GLOBAL_RESPONSE_PROGRESS]?: ProgressCallback;
}
export interface ReqInit extends Omit<RequestInit, 'body'> {
    url?: string;
    method?: 'GET' | 'POST' | 'PUT' | 'DELETE' | 'HEAD' | 'PATCH';
    meta?: Record<string, any>;
    timeout?: number;
    /**
     * This should be consistent with the parsing method of Response
     */
    responseType?: ResponseType;
    throwHttpError?: boolean;
    body?: BodyInit | Record<string, any>;
    onResponseProgress?: ProgressCallback;
    [ON_GLOBAL_RESPONSE_PROGRESS]?: ProgressCallback;
}
export declare type Next = (req: Req) => Promise<Res>;
export declare type Middleware = (next: Next) => (req: Req) => Promise<Res>;
export declare type ResponseType = 'json' | 'arrayBuffer' | 'blob' | 'formData' | 'text' | null | false;
export interface Progress {
    ratio: number;
    carry: number;
    total: number;
}
export declare type ProgressCallback = (progress: Progress, chunk: Uint8Array) => void;
export interface Options extends ReqInit {
    baseUrl?: string;
    params?: Record<string, any>;
}
